// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:async';

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account.dart';
import 'package:wyatt_authentication_bloc/src/features/sign_in/cubit/sign_in_cubit.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

mixin SignInWithGoogle<Extra> on BaseSignInCubit<Extra> {
  FutureOrResult<void> onSignInWithGoogle(
    Result<Account, AppException> result,
    WyattForm form,
  );

  FutureOr<void> signInWithGoogle() async {
    if (state.status.isSubmissionInProgress) {
      return;
    }
    final form = formRepository.accessForm(formName);
    emit(SignInState(form: form, status: FormStatus.submissionInProgress));

    final result = await authenticationRepository.signInWithGoogle();

    // Here custom code
    final callbackResponse = await onSignInWithGoogle(result, form);
    if (callbackResponse.isErr) {
      final error = callbackResponse.err!;
      emit(
        SignInState(
          form: form,
          errorMessage: error.message,
          status: FormStatus.submissionFailure,
        ),
      );
    }

    emit(
      result.fold(
        (value) =>
            SignInState(form: form, status: FormStatus.submissionSuccess),
        (error) => SignInState(
          form: form,
          errorMessage: error.message,
          status: FormStatus.submissionFailure,
        ),
      ),
    );
  }
}
